/*
 * Atmel MultiMedia Card Interface driver
 *
 * Copyright (C) 2004-2006 Atmel Corporation
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 */
#ifndef __DRIVERS_MMC_ATMEL_MCI_H__
#define __DRIVERS_MMC_ATMEL_MCI_H__

/* MCI register offsets */
#define MCI_CR					0x0000
#define MCI_MR					0x0004
#define MCI_DTOR				0x0008
#define MCI_SDCR				0x000c
#define MCI_ARGR				0x0010
#define MCI_CMDR				0x0014
#define MCI_BLKR				0x0018
#define MCI_RSPR				0x0020
#define MCI_RSPR1				0x0024
#define MCI_RSPR2				0x0028
#define MCI_RSPR3				0x002c
#define MCI_RDR					0x0030
#define MCI_TDR					0x0034
#define MCI_SR					0x0040
#define MCI_IER					0x0044
#define MCI_IDR					0x0048
#define MCI_IMR					0x004c

/* Bitfields in CR */
#define MCI_MCIEN_OFFSET			0
#define MCI_MCIEN_SIZE				1
#define MCI_MCIDIS_OFFSET			1
#define MCI_MCIDIS_SIZE				1
#define MCI_PWSEN_OFFSET			2
#define MCI_PWSEN_SIZE				1
#define MCI_PWSDIS_OFFSET			3
#define MCI_PWSDIS_SIZE				1
#define MCI_SWRST_OFFSET			7
#define MCI_SWRST_SIZE				1

/* Bitfields in MR */
#define MCI_CLKDIV_OFFSET			0
#define MCI_CLKDIV_SIZE				8
#define MCI_PWSDIV_OFFSET			8
#define MCI_PWSDIV_SIZE				3
#define MCI_RDPROOF_OFFSET			11
#define MCI_RDPROOF_SIZE			1
#define MCI_WRPROOF_OFFSET			12
#define MCI_WRPROOF_SIZE			1
#define MCI_DMAPADV_OFFSET			14
#define MCI_DMAPADV_SIZE			1
#define MCI_BLKLEN_OFFSET			16
#define MCI_BLKLEN_SIZE				16

/* Bitfields in DTOR */
#define MCI_DTOCYC_OFFSET			0
#define MCI_DTOCYC_SIZE				4
#define MCI_DTOMUL_OFFSET			4
#define MCI_DTOMUL_SIZE				3

/* Bitfields in SDCR */
#define MCI_SDCSEL_OFFSET			0
#define MCI_SDCSEL_SIZE				4
#define MCI_SDCBUS_OFFSET			7
#define MCI_SDCBUS_SIZE				1

/* Bitfields in ARGR */
#define MCI_ARG_OFFSET				0
#define MCI_ARG_SIZE				32

/* Bitfields in CMDR */
#define MCI_CMDNB_OFFSET			0
#define MCI_CMDNB_SIZE				6
#define MCI_RSPTYP_OFFSET			6
#define MCI_RSPTYP_SIZE				2
#define MCI_SPCMD_OFFSET			8
#define MCI_SPCMD_SIZE				3
#define MCI_OPDCMD_OFFSET			11
#define MCI_OPDCMD_SIZE				1
#define MCI_MAXLAT_OFFSET			12
#define MCI_MAXLAT_SIZE				1
#define MCI_TRCMD_OFFSET			16
#define MCI_TRCMD_SIZE				2
#define MCI_TRDIR_OFFSET			18
#define MCI_TRDIR_SIZE				1
#define MCI_TRTYP_OFFSET			19
#define MCI_TRTYP_SIZE				2

/* Bitfields in BLKR */
#define MCI_BCNT_OFFSET				0
#define MCI_BCNT_SIZE				16

/* Bitfields in RSPRn */
#define MCI_RSP_OFFSET				0
#define MCI_RSP_SIZE				32

/* Bitfields in SR/IER/IDR/IMR */
#define MCI_CMDRDY_OFFSET			0
#define MCI_CMDRDY_SIZE				1
#define MCI_RXRDY_OFFSET			1
#define MCI_RXRDY_SIZE				1
#define MCI_TXRDY_OFFSET			2
#define MCI_TXRDY_SIZE				1
#define MCI_BLKE_OFFSET				3
#define MCI_BLKE_SIZE				1
#define MCI_DTIP_OFFSET				4
#define MCI_DTIP_SIZE				1
#define MCI_NOTBUSY_OFFSET			5
#define MCI_NOTBUSY_SIZE			1
#define MCI_ENDRX_OFFSET			6
#define MCI_ENDRX_SIZE				1
#define MCI_ENDTX_OFFSET			7
#define MCI_ENDTX_SIZE				1
#define MCI_RXBUFF_OFFSET			14
#define MCI_RXBUFF_SIZE				1
#define MCI_TXBUFE_OFFSET			15
#define MCI_TXBUFE_SIZE				1
#define MCI_RINDE_OFFSET			16
#define MCI_RINDE_SIZE				1
#define MCI_RDIRE_OFFSET			17
#define MCI_RDIRE_SIZE				1
#define MCI_RCRCE_OFFSET			18
#define MCI_RCRCE_SIZE				1
#define MCI_RENDE_OFFSET			19
#define MCI_RENDE_SIZE				1
#define MCI_RTOE_OFFSET				20
#define MCI_RTOE_SIZE				1
#define MCI_DCRCE_OFFSET			21
#define MCI_DCRCE_SIZE				1
#define MCI_DTOE_OFFSET				22
#define MCI_DTOE_SIZE				1
#define MCI_OVRE_OFFSET				30
#define MCI_OVRE_SIZE				1
#define MCI_UNRE_OFFSET				31
#define MCI_UNRE_SIZE				1

/* Constants for DTOMUL */
#define MCI_DTOMUL_1_CYCLE			0
#define MCI_DTOMUL_16_CYCLES			1
#define MCI_DTOMUL_128_CYCLES			2
#define MCI_DTOMUL_256_CYCLES			3
#define MCI_DTOMUL_1024_CYCLES			4
#define MCI_DTOMUL_4096_CYCLES			5
#define MCI_DTOMUL_65536_CYCLES			6
#define MCI_DTOMUL_1048576_CYCLES		7

/* Constants for RSPTYP */
#define MCI_RSPTYP_NO_RESP			0
#define MCI_RSPTYP_48_BIT			1
#define MCI_RSPTYP_136_BIT			2

/* Constants for SPCMD */
#define MCI_SPCMD_NO_SPEC_CMD			0
#define MCI_SPCMD_INIT_CMD			1
#define MCI_SPCMD_SYNC_CMD			2
#define MCI_SPCMD_INT_CMD			4
#define MCI_SPCMD_INT_RESP			5

/* Constants for TRCMD */
#define MCI_TRCMD_NO_TRANS			0
#define MCI_TRCMD_START_TRANS			1
#define MCI_TRCMD_STOP_TRANS			2

/* Constants for TRTYP */
#define MCI_TRTYP_BLOCK				0
#define MCI_TRTYP_MULTI_BLOCK			1
#define MCI_TRTYP_STREAM			2

/* Bit manipulation macros */
#define MCI_BIT(name)					\
	(1 << MCI_##name##_OFFSET)
#define MCI_BF(name,value)				\
	(((value) & ((1 << MCI_##name##_SIZE) - 1))	\
	 << MCI_##name##_OFFSET)
#define MCI_BFEXT(name,value)				\
	(((value) >> MCI_##name##_OFFSET)		\
	 & ((1 << MCI_##name##_SIZE) - 1))
#define MCI_BFINS(name,value,old)			\
	(((old) & ~(((1 << MCI_##name##_SIZE) - 1)	\
		    << MCI_##name##_OFFSET))		\
	 | MCI_BF(name,value))

/* Register access macros */
#define mci_readl(port,reg)				\
	__raw_readl((port)->regs + MCI_##reg)
#define mci_writel(port,reg,value)			\
	__raw_writel((value), (port)->regs + MCI_##reg)

#endif /* __DRIVERS_MMC_ATMEL_MCI_H__ */
